﻿using System.Collections;
using System.ComponentModel;
using System.Configuration.Install;
using System.ServiceProcess;

namespace DotNet.Common.WinSvcInstaller
{
    /// <summary>
    /// This is a custom project installer.
    /// Applies a unique name to the service using the /name switch
    /// Sets description to the service using the /desc switch
    /// Sets displayname to the service using the /displayname switch
    /// Sets user name and password using the /user and /password switches
    /// Allows the use of a local account using the /account switch
    /// </summary>
    [RunInstaller(true)]
    public partial class DynamicInstaller : Installer
    {
        public string ServiceName
        {
            get { return serviceInstaller.ServiceName; }
            set { serviceInstaller.ServiceName = value; }
        }
        public string DisplayName
        {
            get { return serviceInstaller.DisplayName; }
            set { serviceInstaller.DisplayName = value; }
        }
        public string Description
        {
            get { return serviceInstaller.Description; }
            set { serviceInstaller.Description = value; }
        }
        public ServiceStartMode StartType
        {
            get { return serviceInstaller.StartType; }
            set { serviceInstaller.StartType = value; }
        }
        public ServiceAccount Account
        {
            get { return processInstaller.Account; }
            set { processInstaller.Account = value; }
        }
        public string ServiceUsername
        {
            get { return processInstaller.Username; }
            set { processInstaller.Username = value; }
        }
        public string ServicePassword
        {
            get { return processInstaller.Password; }
            set { processInstaller.Password = value; }
        }

        private ServiceProcessInstaller processInstaller = null;
        private System.ServiceProcess.ServiceInstaller serviceInstaller = null;

        public DynamicInstaller()
        {
            processInstaller = new ServiceProcessInstaller();
            processInstaller.Account = ServiceAccount.LocalService;
            processInstaller.Username = null;
            processInstaller.Password = null;
            serviceInstaller = new System.ServiceProcess.ServiceInstaller();
            serviceInstaller.StartType = ServiceStartMode.Manual;
            serviceInstaller.ServiceName = "DynamicService";
            serviceInstaller.DisplayName = "DynamicService";
            serviceInstaller.Description = "Dynamic Service";

            Installers.AddRange(new Installer[] {
            processInstaller,
            serviceInstaller});
        }

        #region Access parameters

        /// <summary>
        /// Return the value of the parameter in dicated by key
        /// </summary>
        /// <PARAM name="key">Context parameter key</PARAM>
        /// <returns>Context parameter specified by key</returns>
        public string GetContextParameter(string key)
        {
            string result = string.Empty;
            if (this.Context.Parameters.ContainsKey(key) == true)
            {
                result = this.Context.Parameters[key];
            }
            return result;
        }

        #endregion

        /// <summary>
        /// This method is run before the install process.
        /// This method is overriden to set the following parameters:
        /// service name (/name switch)
        /// service description (/desc switch)
        /// account type (/account switch)
        /// for a user account user name (/user switch)
        /// for a user account password (/password switch)
        /// Note that when using a user account,
        /// if the user name or password is not set,
        /// the installing user is prompted for the credentials to use.
        /// </summary>
        /// <PARAM name="savedState"></PARAM>
        protected override void OnBeforeInstall(IDictionary savedState)
        {
            base.OnBeforeInstall(savedState);

            bool isUserAccount = false;

            // Decode the command line switches
            string name = GetContextParameter("name").Trim();
            if (string.IsNullOrEmpty(name) == false)
            {
                serviceInstaller.ServiceName = name;
            }
            string displayName = GetContextParameter("displayname").Trim();
            if (string.IsNullOrEmpty(displayName) == false)
            {
                serviceInstaller.DisplayName = displayName;
            }
            string desc = GetContextParameter("desc").Trim();
            if (string.IsNullOrEmpty(desc) == false)
            {
                serviceInstaller.Description = desc;
            }

            // What start type of the service
            string startType = GetContextParameter("starttype");
            switch (startType.ToLower())
            {
                default:
                    break;
                case "automatic":
                    serviceInstaller.StartType = ServiceStartMode.Automatic;
                    break;
                case "manual":
                    serviceInstaller.StartType = ServiceStartMode.Manual;
                    break;
                case "disabled":
                    serviceInstaller.StartType = ServiceStartMode.Disabled;
                    break;
            }

            // What type of credentials to use to run the service
            string acct = GetContextParameter("account");
            switch (acct.ToLower())
            {
                default:
                    break;
                case "user":
                    processInstaller.Account = ServiceAccount.User;
                    isUserAccount = true;
                    break;
                case "localservice":
                    processInstaller.Account = ServiceAccount.LocalService;
                    break;
                case "localsystem":
                    processInstaller.Account = ServiceAccount.LocalSystem;
                    break;
                case "networkservice":
                    processInstaller.Account = ServiceAccount.NetworkService;
                    break;
            }

            // User name and password
            string username = GetContextParameter("user").Trim();
            string password = GetContextParameter("password").Trim();

            // Should I use a user account?
            if (isUserAccount==true)
            {
                // If we need to use a user account,
                // set the user name and password
                if (string.IsNullOrEmpty(username) == false)
                {
                    processInstaller.Username = username;
                }
                if (string.IsNullOrEmpty(password) == false)
                {
                    processInstaller.Password = password;
                }
            }
        }

        /// <summary>
        /// Uninstall based on the service name
        /// </summary>
        /// <PARAM name="savedState"></PARAM>
        protected override void OnBeforeUninstall(IDictionary savedState)
        {
            base.OnBeforeUninstall(savedState);

            // Set the service name based on the command line
            string name = GetContextParameter("name").Trim();
            if (string.IsNullOrEmpty(name) == false)
            {
                serviceInstaller.ServiceName = name;
            }
        }
    }

}
