﻿using System;
using System.Diagnostics;
using System.IO;

namespace DotNet.Common.WinSvcInstaller
{

    /// <summary>
    /// provided url: http://www.codeproject.com/KB/cs/DynWinServiceInstallUtil.aspx
    /// To use this class for installing windows service, you need a project installer (which inherits from DynamicInstaller) added to your assembly
    /// </summary>
    public class WindowsServiceInstallUtil
    {
        /// <summary>
        /// Path to folder where InstallUtil (.Net SDK) stored
        /// </summary>
        public static string InstallUtilPath = System.Runtime.InteropServices.RuntimeEnvironment.GetRuntimeDirectory(); //@"C:\WINDOWS\Microsoft.NET\Framework\v2.0.50727";

        protected WindowsServiceInstallInfo _wsInstallInfo = null;

        public WindowsServiceInstallUtil(WindowsServiceInstallInfo wsInstallInfo)
        {
            _wsInstallInfo = wsInstallInfo;
        }


        /// <summary>
        /// Run InstallUtil with specific params
        /// </summary>
        /// <param name="installUtilArguments">CommandLine params</param>
        /// <returns>Status of installation</returns>
        private static bool CallInstallUtil(string installUtilArguments)
        {
            Process proc = new Process();
            proc.StartInfo.FileName = Path.Combine(InstallUtilPath, "installutil.exe");
            proc.StartInfo.Arguments = installUtilArguments;
            proc.StartInfo.WindowStyle = ProcessWindowStyle.Hidden;//hide a window
            proc.StartInfo.RedirectStandardOutput = true;
            proc.StartInfo.UseShellExecute = false;

            proc.Start();
            string outputResult = proc.StandardOutput.ReadToEnd();
            proc.WaitForExit();

            // check result
            if (proc.ExitCode != 0)
            {
                //Console.WriteLine("{0} : failed with code {1}", DateTime.Now, proc.ExitCode);
                //Console.WriteLine(outputResult);
                return false;
            }
            return true;
        }

        /// <summary>
        /// Install windows service
        /// </summary>
        /// <returns></returns>
        public bool Install()
        {
            return Install(string.Empty);
        }

        /// <summary>
        /// Install windows service
        /// </summary>
        /// <param name="logFilePath">Log file to store installation information</param>
        /// <returns></returns>
        public virtual bool Install(string logFilePath)
        {
            string installUtilArguments = GenerateInstallutilInstallArgs(logFilePath);
            return CallInstallUtil(installUtilArguments);
        }

        protected string GenerateInstallutilInstallArgs(string logFilePath)
        {
            string installUtilArguments = " /account=\"" + _wsInstallInfo.WsAccountType + "\"";
            if (string.IsNullOrEmpty(_wsInstallInfo.WindowsServiceName) == false)
            {
                installUtilArguments += " /name=\"" + _wsInstallInfo.WindowsServiceName + "\"";
            }
            if (string.IsNullOrEmpty(_wsInstallInfo.WinSvcDisplayName) == false)
            {
                installUtilArguments += " /displayname=\"" + _wsInstallInfo.WinSvcDisplayName + "\"";
            }
            if (string.IsNullOrEmpty(_wsInstallInfo.Description) == false)
            {
                installUtilArguments += " /desc=\"" + _wsInstallInfo.Description + "\"";
            }
            if (string.IsNullOrEmpty(_wsInstallInfo.WsStartType) == false)
            {
                installUtilArguments += " /starttype=\"" + _wsInstallInfo.WsStartType + "\"";
            }
            if (_wsInstallInfo.WsAccountType == WindowsServiceAccountType.User)
            {
                installUtilArguments += " /user=\"" + _wsInstallInfo.WsAccountUserName + "\" /password=\"" + _wsInstallInfo.WsAccountPassword + "\"";
            }
            installUtilArguments += " \"" + Path.Combine(_wsInstallInfo.WindowsServicePath, _wsInstallInfo.WindowsServiceAssemblyName) + "\"";
            if (string.IsNullOrEmpty(logFilePath.Trim()) == false)
            {
                installUtilArguments += " /LogFile=\"" + logFilePath + "\"";
            }
            return installUtilArguments;
        }

        /// <summary>
        /// Uninstall windows service
        /// </summary>
        /// <returns></returns>
        public bool Uninstall()
        {
            return Uninstall(string.Empty);
        }

        /// <summary>
        /// Uninstall windows service
        /// </summary>
        /// <param name="logFilePath">Log file to store installation information</param>
        /// <returns></returns>
        public virtual bool Uninstall(string logFilePath)
        {
            string installUtilArguments = GenerateInstallutilUninstallArgs(logFilePath);
            return CallInstallUtil(installUtilArguments);
        }

        protected string GenerateInstallutilUninstallArgs(string logFilePath)
        {
            string installUtilArguments = " /u ";
            if (string.IsNullOrEmpty(_wsInstallInfo.WindowsServiceName) == false)
            {
                installUtilArguments += " /name=\"" + _wsInstallInfo.WindowsServiceName + "\"";
            }
            installUtilArguments += " \"" + Path.Combine(_wsInstallInfo.WindowsServicePath, _wsInstallInfo.WindowsServiceAssemblyName) + "\"";
            if (string.IsNullOrEmpty(logFilePath.Trim()) == false)
            {
                installUtilArguments += " /LogFile=\"" + logFilePath + "\"";
            }

            return installUtilArguments;
        }
    }

    /// <summary>
    /// win svc服务账户类型枚举
    /// </summary>
    public enum WindowsServiceAccountType
    {
        LocalService,
        NetworkService,
        LocalSystem,
        User
    }

    /// <summary>
    /// installer需要的相关配置信息，如服务名称，服务介绍，用户名，密码等等
    /// </summary>
    public class WindowsServiceInstallInfo
    {
        private string _windowsServiceName;
        private string _windowsServiceDisplayName;
        private string _wsDescription;
        private readonly string _windowsServicePath;
        private readonly string _windowsServiceAssemblyName;
        private readonly WindowsServiceAccountType _wsAccountType;
        private readonly string _wsAccountUserName = string.Empty;
        private readonly string _wsAccountPassword = string.Empty;
        private readonly string _wsStartType = "Automatic";

        /// <summary>
        /// Please pass the windows service information in
        /// </summary>
        /// <param name="windowsServiceName">Name of windows service</param>
        /// <param name="description">Description of windows service</param>
        /// <param name="windowsServicePath">Path to folder where windows service assembly stored</param>
        /// <param name="windowsServiceAssemblyName">Windows Service Assembly Name</param>
        /// <param name="wsAccountType">Windows Service Account Type</param>
        /// <param name="wsAccountUserName">Username of Windows Service when Account Type is USER</param>
        /// <param name="wsAccountPassword">Password of Windows Service when Account Type is USER</param>
        /// <param name="wsStartType">Windows Service Start Type</param>
        public WindowsServiceInstallInfo(string windowsServiceName, string displayName, string description, string windowsServicePath, string windowsServiceAssemblyName, WindowsServiceAccountType wsAccountType, string wsAccountUserName, string wsAccountPassword, string wsStartType)
        {
            _windowsServiceName = windowsServiceName.Trim();
            _windowsServiceDisplayName = displayName.Trim();
            _wsDescription = description.Trim();
            _windowsServicePath = windowsServicePath;
            _windowsServiceAssemblyName = windowsServiceAssemblyName;
            _wsAccountType = wsAccountType;
            _wsAccountUserName = wsAccountUserName;
            _wsAccountPassword = wsAccountPassword;
            _wsStartType = wsStartType;

            if (_wsAccountType == WindowsServiceAccountType.User && string.IsNullOrEmpty(_wsAccountUserName) == true)
            {
                throw new Exception("Username has to be provided if AccountType to start the windows service is USER");
            }
        }

        /// <summary>
        /// 卸载专用
        /// </summary>
        /// <param name="windowsServiceName"></param>
        public WindowsServiceInstallInfo(string windowsServiceName, string windowsServicePath, string windowsServiceAssemblyName)
        {
            _windowsServiceName = windowsServiceName.Trim();
            _windowsServiceName = windowsServiceName.Trim();
            _windowsServicePath = windowsServicePath;
            _windowsServiceAssemblyName = windowsServiceAssemblyName;
        }

        /// <summary>
        /// 常用配置(默认用户名和密码都为空)
        /// </summary>
        /// <param name="windowsServiceName">服务名</param>
        /// <param name="displayName">服务显示名</param>
        /// <param name="description">服务简介</param>
        /// <param name="windowsServicePath">服务所在文件夹路径</param>
        /// <param name="windowsServiceAssemblyName">服务对应exe文件名称</param>
        public WindowsServiceInstallInfo(string windowsServiceName, string displayName, string description, string windowsServicePath, string windowsServiceAssemblyName,string startType)
            : this(windowsServiceName, displayName, description, windowsServicePath, windowsServiceAssemblyName, WindowsServiceAccountType.LocalSystem, string.Empty, string.Empty,startType) { }

        /// <summary>
        /// 常用配置(可配置账户类型，用户名和密码)
        /// </summary>
        /// <param name="windowsServiceName"></param>
        /// <param name="displayName"></param>
        /// <param name="windowsServicePath"></param>
        /// <param name="windowsServiceAssemblyName"></param>
        /// <param name="accountType"></param>
        /// <param name="wsAccountUserName"></param>
        /// <param name="wsAccountPassword"></param>
        public WindowsServiceInstallInfo(string windowsServiceName, string displayName, string windowsServicePath, string windowsServiceAssemblyName, WindowsServiceAccountType accountType, string wsAccountUserName, string wsAccountPassword,string startType)
            : this(windowsServiceName, displayName, displayName, windowsServicePath, windowsServiceAssemblyName, accountType, wsAccountUserName, wsAccountPassword,startType) { }

        /// <summary>
        /// 服务名称
        /// </summary>
        public string WindowsServiceName
        {
            get { return _windowsServiceName; }
            set { _windowsServiceName = value; }
        }

        /// <summary>
        /// 服务显示名称
        /// </summary>
        public string WinSvcDisplayName
        {
            get { return _windowsServiceDisplayName; }
            set { _windowsServiceDisplayName = value; }
        }

        /// <summary>
        /// Description of windows service
        /// </summary>
        public string Description
        {
            get { return _wsDescription; }
            set { _wsDescription = value; }
        }

        /// <summary>
        /// Path to folder which contains windows service binary
        /// </summary>
        public string WindowsServicePath
        {
            get { return _windowsServicePath; }
        }
        /// <summary>
        /// Windows service binary file name
        /// </summary>
        public string WindowsServiceAssemblyName
        {
            get { return _windowsServiceAssemblyName; }
        }
        /// <summary>
        /// Account type to start windows service
        /// </summary>
        public WindowsServiceAccountType WsAccountType
        {
            get { return _wsAccountType; }
        }
        /// <summary>
        /// Username to start windows service (if account type is User)
        /// </summary>
        public string WsAccountUserName
        {
            get { return _wsAccountUserName; }
        }
        /// <summary>
        /// Password of username to start windows service (if account type is User)
        /// </summary>
        public string WsAccountPassword
        {
            get { return _wsAccountPassword; }
        }

        /// <summary>
        /// 启动类型
        /// </summary>
        public string WsStartType
        {
            get { return _wsStartType; }
        } 

    }

}
